import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import {DatePipe} from "@angular/common";
import {MatTableDataSource} from "@angular/material/table";
import {PartitionAssistantComponent} from "./partition-assistant/partition-assistant.component";
import {MatDialog} from "@angular/material/dialog";
import {Router} from "@angular/router";

interface ClientInfo {
  property: string;
  value: any;
}

@Component({
  selector: 'app-client-main-view',
  templateUrl: './client-main-view.component.html',
  styleUrl: './client-main-view.component.css'
})
export class ClientMainViewComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  @ViewChild('assistantContainer') assistantContainer!: ElementRef;
  clientUuid: string;
  clientData: any = {};
  isPartitionAssistantVisible: boolean = false;
  isBootImageVisible: boolean = false;
  isDiskUsageVisible: boolean = true;
  dataSource = new MatTableDataSource<any>();
  generalData: ClientInfo[] = [];
  networkData: ClientInfo[] = [];
  classroomData: ClientInfo[] = [];
  diskUsageData: any[] = [];
  partitions: any[] = [];
  commands: any[] = [];
  datePipe: DatePipe = new DatePipe('es-ES');
  columns = [
    {
      columnDef: 'diskNumber',
      header: 'Disco',
      cell: (partition: any) => `${partition.diskNumber}`,
    },
    {
      columnDef: 'partitionNumber',
      header: 'Particion',
      cell: (partition: any) => `${partition.partitionNumber}`
    },
    {
      columnDef: 'description',
      header: 'Sistema de ficheros',
      cell: (partition: any) => `${partition.filesystem}`
    },
    {
      columnDef: 'size',
      header: 'Tamaño',
      cell: (partition: any) => `${(partition.size / 1024).toFixed(2)} GB`
    },
    {
      columnDef: 'memoryUsage',
      header: 'Uso',
      cell: (partition: any) => `${partition.memoryUsage} %`
    },
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef)];
  isDiskUsageEmpty: boolean = true;
  loading: boolean = true;

  constructor(
    private http: HttpClient,
    private dialog: MatDialog,
    private router: Router
    ) {
    const url = window.location.href;
    const segments = url.split('/');
    this.clientUuid = segments[segments.length - 1];
  }

  ngOnInit() {
    this.clientData = history.state.clientData;
    this.loadPartitions()
    this.updateGeneralData();
    this.updateNetworkData();
    this.loadCommands()
    this.calculateDiskUsage();
    this.loading = false;
  }

  updateGeneralData() {
    this.generalData = [
      { property: 'Nombre', value: this.clientData?.name || '' },
      { property: 'Uuid', value: this.clientData?.uuid || '' },
      { property: 'IP', value: this.clientData?.ip || '' },
      { property: 'MAC', value: this.clientData?.mac || '' },
      { property: 'Nº de serie', value: this.clientData?.serialNumber || '' },
      { property: 'Netiface', value: this.clientData?.netiface || '' },
      { property: 'Perfil hardware', value: this.clientData?.hardwareProfile?.description || '' },
      { property: 'Menú', value: this.clientData?.menu?.description || '' },
    ];
  }

  updateNetworkData() {
    this.networkData = [
      { property: 'Remote Pc', value: this.clientData.remotePc || '' },
      { property: 'Subred', value: this.clientData?.subnet || '' },
      { property: 'OGlive', value: '' },
      { property: 'Autoexec', value: '' },
      { property: 'Repositorio', value: '' },
      { property: 'Validación', value: this.clientData?.organizationalUnit?.networkSettings?.validation || '' },
      { property: 'Pxe', value: this.clientData?.template.name || '' },
      { property: 'Creado por', value: this.clientData?.createdBy || '' }
    ];
  }

  calculateDiskUsage() {
    const diskUsageMap = new Map<number, { total: number, used: number, partitions: any[] }>();

    this.partitions.forEach((partition: any) => {
      const diskNumber = partition.diskNumber;

      if (!diskUsageMap.has(diskNumber)) {
        diskUsageMap.set(diskNumber, { total: 0, used: 0, partitions: [] });
      }

      const diskData = diskUsageMap.get(diskNumber);

      if (partition.partitionNumber === 0) {
        diskData!.total = Number((partition.size / 1024).toFixed(2));
      } else {
        diskData!.used += Number(((partition.size * (partition.memoryUsage / 100)) / 1024).toFixed(2));
        diskData!.partitions.push(partition);
      }
    });

    this.diskUsageData = Array.from(diskUsageMap.entries()).map(([diskNumber, { total, used, partitions }]) => {
      const percentage = total > 0 ? Math.round((used / total) * 100) : 0;
      return { diskNumber, total, used, percentage, partitions };
    });

    this.isDiskUsageEmpty = this.diskUsageData.length === 0;
  }

  getStrokeOffset(partitions: any[], index: number): number {
    const totalSize = partitions.reduce((acc, part) => acc + (part.size / 1024), 0);

    const offset = partitions.slice(0, index).reduce((acc, part) => acc + (part.size / 1024), 0);
    console.log(offset, totalSize)

    return totalSize > 0 ? (offset / totalSize)  : 0;
  }

  loadPartitions(): void {
    this.http.get<any>(`${this.baseUrl}/partitions?client.id=${this.clientData?.id}`).subscribe({
      next: data => {
        this.dataSource = data['hydra:member'];
        this.partitions = data['hydra:member'];
        this.calculateDiskUsage();
      },
      error: error => {
        console.error('Error al obtener las particiones:', error);
      }
    });
  }

  loadCommands(): void {
    this.http.get<any>(`${this.baseUrl}/commands?`).subscribe({
      next: data => {
        this.commands = data['hydra:member'];
      },
      error: error => {
        console.error('Error al obtener las particiones:', error);
      }
    });
  }

  onCommandSelect(command: any): void {
    if (command.name === 'Particionar y Formatear') {
      this.openPartitionAssistant();
    }
  }

  openPartitionAssistant(): void {
    console.log(this.clientData)
    this.router.navigate([`/client/${this.clientData.uuid}/partition-assistant`]).then(r => {
      console.log('navigated', r);
    });
  }
}
