import { Component } from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {DatePipe} from "@angular/common";
import {DataService} from "../client-tab-view/data.service";
import {MatDialog} from "@angular/material/dialog";
import {ToastrService} from "ngx-toastr";
import {HttpClient} from "@angular/common/http";
import {EditClientComponent} from "../../shared/clients/edit-client/edit-client.component";
import {CreateClientComponent} from "../../shared/clients/create-client/create-client.component";
import {DeleteModalComponent} from "../../../../shared/delete_modal/delete-modal/delete-modal.component";
import {catchError} from "rxjs/operators";
import {throwError} from "rxjs";
import {
  CreateOrganizationalUnitComponent
} from "../../shared/organizational-units/create-organizational-unit/create-organizational-unit.component";
import {
  ShowOrganizationalUnitComponent
} from "../../shared/organizational-units/show-organizational-unit/show-organizational-unit.component";
import {
  EditOrganizationalUnitComponent
} from "../../shared/organizational-units/edit-organizational-unit/edit-organizational-unit.component";
import {ClassroomViewDialogComponent} from "../../shared/classroom-view/classroom-view-modal";
import {TreeViewComponent} from "../../shared/tree-view/tree-view.component";

@Component({
  selector: 'app-organizational-unit-tab-view',
  templateUrl: './organizational-unit-tab-view.component.html',
  styleUrl: './organizational-unit-tab-view.component.css'
})
export class OrganizationalUnitTabViewComponent {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  dataSource = new MatTableDataSource<any>();
  length: number = 0;
  loading:boolean = false;
  itemsPerPage: number = 10;
  pageSizeOptions: number[] = [5, 10, 25, 100];
  page: number = 0;
  filters: { [key: string]: string } = {};
  organizationalUnits: any[] = [];
  datePipe: DatePipe = new DatePipe('es-ES');

  private apiUrl = `${this.baseUrl}/organizational-units`;

  columns = [
    {
      columnDef: 'id',
      header: 'ID',
      cell: (ou: any) => `${ou.id}`
    },
    {
      columnDef: 'name',
      header: 'Nombre',
      cell: (ou: any) => `${ou.name}`
    },
    {
      columnDef: 'type',
      header: 'Tipo',
      cell: (ou: any) => `${ou.type}`
    },
    {
      columnDef: 'remoteCalendar',
      header: 'Calendario',
      cell: (ou: any) => `${ou.remoteCalendar ? ou.remoteCalendar.name : '-'}`
    },
    {
      columnDef: 'available',
      header: 'Disponible remotePC',
      cell: (ou: any) => `${ou.available ? 'No' : 'Si'}`
    },
    {
      columnDef: 'createdAt',
      header: 'Fecha de creación',
      cell: (ou: any) => `${this.datePipe.transform(ou.createdAt, 'dd/MM/yyyy hh:mm:ss')}`
    }
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef), 'actions'];

  constructor(
    private dataService: DataService,
    public dialog: MatDialog,
    private toastService: ToastrService,
    private http: HttpClient
  ) {}

  ngOnInit(): void {
    this.getOrganizationalUnits();
  }

  roomMap(room: any): void {
    console.log(room)
    const dialogRef = this.dialog.open(ClassroomViewDialogComponent, {
      width: '90vw',
      data: { clients: room.clients }
    });
  }

  onTreeClick(data: any): void {
    const dialogRef = this.dialog.open(TreeViewComponent, { data: { data }, width: '800px'});
  }

  getOrganizationalUnits() {
    this.http.get<any>(`${this.apiUrl}?&page=${this.page + 1}&itemsPerPage=${this.itemsPerPage}`, { params: this.filters }).subscribe(
      (data) => {
        this.dataSource.data = data['hydra:member'];
        this.length = data['hydra:totalItems'];
      },
      (error) => {
        console.error('Error fetching ou', error);
      }
    );
  }

  onShowClick(event: MouseEvent, data: any): void {
    event.stopPropagation();
    if (data.type != "client") {
      const dialogRef = this.dialog.open(ShowOrganizationalUnitComponent, { data: { data }, width: '700px'});
    }
  }

  onEditClick(event: MouseEvent, uuid: string): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(EditOrganizationalUnitComponent, { data: { uuid }, width: '700px'});
    dialogRef.afterClosed().subscribe(() => this.getOrganizationalUnits());
  }

  addOrganizationalUnit(event: MouseEvent, organizationalUnit:any = null): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(CreateOrganizationalUnitComponent, { data: { organizationalUnit }, width: '900px'});
    dialogRef.afterClosed().subscribe(() => {
      this.getOrganizationalUnits();
    });
  }

  onDeleteClick(event: MouseEvent, client: any): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(DeleteModalComponent, {
      width: '400px'
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.http.delete<void>(`${this.apiUrl}/${client.uuid}`).pipe(
          catchError(error => {
            this.toastService.error(error.error['hydra:description']);
            return throwError(error);
          })
        ).subscribe(() => {
          this.toastService.success('Elemento eliminado correctamente');
          this.getOrganizationalUnits();
        });
      }
    });
  }

  resetFilters() {
    this.loading = true;
    this.filters = {};
    this.getOrganizationalUnits();
  }

  search(): void {
    this.loading = true;
    this.getOrganizationalUnits()
  }

  onPageChange(event: any): void {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.length = event.length;
    this.getOrganizationalUnits();
  }
}
