import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatSnackBar } from '@angular/material/snack-bar';
import { ToastrService } from 'ngx-toastr';
import { DataService } from '../../../services/data.service';

@Component({
  selector: 'app-create-client',
  templateUrl: './create-client.component.html',
  styleUrls: ['./create-client.component.css']
})
export class CreateClientComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  clientForm!: FormGroup;
  parentUnits: any[] = [];
  hardwareProfiles: any[] = [];
  ogLives: any[] = [];
  templates: any[] = [];
  private errorForm: boolean = false;
  protected netifaceTypes = [
    { "name": 'Eth0', "value": "eth0" },
    { "name": 'Eth1', "value": "eth1" },
    { "name": 'Eth2', "value": "eth2" },
  ];
  protected netDriverTypes = [
    { "name": 'Generic', "value": "generic" },
  ];
  loading: boolean = false;

  constructor(
    private fb: FormBuilder,
    private dialogRef: MatDialogRef<CreateClientComponent>,
    private http: HttpClient,
    private snackBar: MatSnackBar,
    private toastService: ToastrService,
    private dataService: DataService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) { }

  ngOnInit(): void {
    console.log(this.data);
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadOgLives();
    this.loadPxeTemplates()
    this.clientForm = this.fb.group({
      organizationalUnit: [this.data.organizationalUnit ? this.data.organizationalUnit['@id'] : null, Validators.required],
      name: ['', Validators.required],
      serialNumber: ['', Validators.required],
      netiface: null,
      netDriver: null,
      mac: ['', Validators.required],
      ip: ['', Validators.required],
      template: [null],
      hardwareProfile: [this.data.organizationalUnit && this.data.organizationalUnit.networkSettings && this.data.organizationalUnit.networkSettings.hardwareProfile ? this.data.organizationalUnit.networkSettings.hardwareProfile['@id'] : null],
      ogLive: [null]
    });
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => {
        this.hardwareProfiles = data;
        this.loading = false;
      },
      (error: any) => {
        console.error('Error fetching hardware profiles', error);
        this.loading = false;
      }
    );
  }

  loadParentUnits() {
    this.loading = true;
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching parent units:', error);
        this.loading = false;
      }
    );
  }

  loadOgLives() {

    const url = `${this.baseUrl}/og-lives?page=1&itemsPerPage=30`;
    this.http.get<any>(url).subscribe(
      response => {
        this.ogLives = response['hydra:member'];
      },
      error => {
        console.error('Error fetching ogLives:', error);
      }
    );
  }

  loadPxeTemplates(): void {
    const url = `${this.baseUrl}/pxe-templates?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.templates = response['hydra:member'];
      },
      error => {
        console.error('Error fetching ogLives:', error);
      }
    );
  }

  onSubmit() {
    if (this.clientForm.valid) {
      this.errorForm = false;
      const formData = this.clientForm.value;
      formData.ogLive = formData.ogLive;
      this.http.post(`${this.baseUrl}/clients`, formData).subscribe(
        response => {
          this.dialogRef.close(response);
          this.openSnackBar(false, 'Cliente creado exitosamente');
        },
        error => {
          console.error('Error during POST:', error);
          this.errorForm = true;
          this.openSnackBar(true, 'Error al crear el cliente: ' + error.error['hydra:description']);
        }
      );
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  openSnackBar(isError: boolean, message: string) {
    if (isError) {
      this.toastService.error(' Error al crear el cliente: ' + message, 'Error');
    } else {
      this.toastService.success(message, 'Éxito');
    }
  }
}
