import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Component, EventEmitter, Inject, OnInit, Output } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { CreateOrganizationalUnitComponent } from '../create-organizational-unit/create-organizational-unit.component';
import {DataService} from "../../../services/data.service";
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-edit-organizational-unit',
  templateUrl: './edit-organizational-unit.component.html',
  styleUrl: './edit-organizational-unit.component.css'
})
export class EditOrganizationalUnitComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  classroomInfoFormGroup: FormGroup;
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  parentUnits: any[] = [];
  hardwareProfiles: any[] = [];
  isEditMode: boolean;
  currentCalendar: any = [];
  protected p2pModeOptions = [
    {"name": 'Leecher', "value": "p2p-mode-leecher"},
    {"name": 'Peer', "value": "p2p-mode-peer"},
    {"name": 'Seeder', "value": "p2p-mode-seeder"},
  ];
  protected multicastModeOptions = [
    {"name": 'Half duplex', "value": "half-duplex"},
    {"name": 'Full duplex', "value": "full-duplex"},
  ];
  @Output() unitAdded = new EventEmitter();
  calendars: any;

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient,
    private dataService: DataService,
    private toastService: ToastrService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
    this.isEditMode = !!data?.uuid;

    this.generalFormGroup = this._formBuilder.group({
      name: [null],
      parent: [null],
      description: [null],
      type: [null]
    });

    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [null],
    });

    this.networkSettingsFormGroup = this._formBuilder.group({
      proxy: [null],
      dns: [null],
      netmask: [null],
      router: [null],
      ntp: [null],
      p2pMode: [null],
      p2pTime: [null],
      mcastIp: [null],
      mcastSpeed: [null],
      mcastPort: [null],
      mcastMode: [null],
      menu: [null],
      hardwareProfile: [null],
      validation: [false]
    });

    this.classroomInfoFormGroup = this._formBuilder.group({
      location: [null],
      projector: [false],
      board: [false],
      capacity: [null],
      remoteCalendar: [null]
    });

    if (this.isEditMode) {
      this.loadData(data.uuid);
    }
  }

  ngOnInit() {
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadCalendars();
  }

  loadParentUnits() {
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
      },
      error => {
        console.error('Error fetching parent units:', error);
      }
    );
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => {
        this.hardwareProfiles = data;
      },
      (error: any) => {
        console.error('Error fetching hardware profiles', error);
      }
    );
  }

  loadCalendars() {
    const apiUrl = `${this.baseUrl}/remote-calendars?page=1&itemsPerPage=30`;
    this.http.get<any>(apiUrl).subscribe(
      response => this.calendars = response['hydra:member'],
      error => {
        console.error('Error loading calendars', error);
        this.toastService.error('Error loading current calendar');
      }
    );
  }

  loadCurrentCalendar(uuid: string): void {
    const apiUrl = `${this.baseUrl}/remote-calendars/${uuid}`;
    this.http.get<any>(apiUrl).subscribe(
      response => this.currentCalendar = response,
      error => {
        console.error('Error loading current calendar', error);
        this.toastService.error('Error loading current calendar');
      }
    );}

  onCalendarChange(event: any) {
    const selectedCalendarId = event.value;
    console.log('Selected calendar ID:', selectedCalendarId);
    this.generalFormGroup.value.remoteCalendar = selectedCalendarId;
  }

  loadData(uuid: string) {
    const url = `${this.baseUrl}/organizational-units/${uuid}`;

    this.http.get<any>(url).subscribe(
      data => {
        this.generalFormGroup.patchValue({
          name: data.name,
          parent: data.parent ? data.parent['@id'] : '',
          description: data.description,
          type: data.type
        });
        this.additionalInfoFormGroup.patchValue({
          comments: data.comments
        });
        this.networkSettingsFormGroup.patchValue({
          proxy: data.networkSettings.proxy,
          dns: data.networkSettings.dns,
          netmask: data.networkSettings.netmask,
          router: data.networkSettings.router,
          ntp: data.networkSettings.ntp,
          p2pMode: data.networkSettings.p2pMode,
          p2pTime: data.networkSettings.p2pTime,
          mcastIp: data.networkSettings.mcastIp,
          mcastSpeed: data.networkSettings.mcastSpeed,
          mcastPort: data.networkSettings.mcastPort,
          mcastMode: data.networkSettings.mcastMode,
          menu: data.networkSettings.menu ? data.networkSettings.menu['@id'] : null,
          hardwareProfile: data.networkSettings.hardwareProfile ? data.networkSettings.hardwareProfile['@id'] : null,
          validation: data.networkSettings.validation
        });
        this.classroomInfoFormGroup.patchValue({
          location: data.location,
          projector: data.projector,
          board: data.board,
          capacity: data.capacity,
          remoteCalendar: data.remoteCalendar ? data.remoteCalendar['@id'] : null
        });
      },

      error => {
        console.error('Error fetching data for edit:', error);
        this.toastService.error('Error fetching data');
        this.onNoClick()
      }
    );
    console.log(this.classroomInfoFormGroup.value);
  }

  onSubmit() {
    if (this.generalFormGroup.valid && this.additionalInfoFormGroup.valid && this.networkSettingsFormGroup.valid) {
      const parentValue = this.generalFormGroup.value.parent;
      const formData = {
        name: this.generalFormGroup.value.name,
        parent: parentValue || null,
        description: this.generalFormGroup.value.description,
        comments: this.additionalInfoFormGroup.value.comments,
        remoteCalendar: this.generalFormGroup.value.remoteCalendar,
        type: this.generalFormGroup.value.type,
        networkSettings: this.networkSettingsFormGroup.value
      };


      if (this.isEditMode) {
        const putUrl = `${this.baseUrl}/organizational-units/${this.data.uuid}`;
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        console.log('PUT URLLLLL:', formData);
        this.http.put<any>(putUrl, formData, { headers }).subscribe(
          response => {
            console.log('PUT successful:', response);
            this.unitAdded.emit();
            this.dialogRef.close();
            this.toastService.success('Editado exitosamente', 'Éxito');
          },
          error => {
            console.error('Error al realizar PUT:', error);
            this.toastService.error('Error al editar:', error);
          }
        );
      } else {
        const postUrl = `${this.baseUrl}/organizational-units`;
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        this.http.post<any>(postUrl, formData, { headers }).subscribe(
          response => {
            this.unitAdded.emit();
            this.dialogRef.close();
            this.toastService.success('Editado exitosamente', 'Éxito');
          },
          error => {
            console.error('Error al realizar POST:', error);
            this.toastService.error('Error al editar:', error);
          }
        );
      }
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  
}
